@@ -54,12 +54,13 @@ get_order.integer <- function(x, ...) {
   o
 }
 
+## returns the order of objects (index of first, second, etc. object)
 get_order.default <- function(x, ...) 
   stop(gettextf("No permutation accessor implemented for class '%s'. ",
     class(x)))
 
-
-#get_rank <- function(x, ...) order(get_order(x, ...))
+## returns for each object its rank (rank of first, second, etc. object)
+get_rank <- function(x, ...) order(get_order(x, ...))
 
 get_permutation_matrix <- function(x, ...) 
   permutation_vector2matrix(get_order(x, ...))
@@ -147,4 +148,4 @@ summary.ser_permutation_vector <- function(object, ...) {
 .valid_permutation_matrix <- function(x) {
   if(any(rowSums(x)!=1) || any(colSums(x)!=1) || any(x!=1 & x!=0)) 
     stop("Not a valid permutation matrix")
-}
\ No newline at end of file
+}
