@@ -22,7 +22,7 @@
 
 seriate.dist <- function(x, method = "ARSA", control = NULL, ...) {
     if(!all(x>=0)) stop("Negative distances not supported!")
-  
+
     ## add ... to control
     control <- c(control, list(...))
 
@@ -34,11 +34,11 @@ seriate.dist <- function(x, method = "ARSA", control = NULL, ...) {
       stop("Argument 'method' must be a character string.")
     method <- get_seriation_method("dist", method)
 
-    if(!is.null(control$verbose) && control$verbose) cat(method$name, ": ", 
+    if(!is.null(control$verbose) && control$verbose) cat(method$name, ": ",
       method$description, "\n", sep="")
-    
+
     order <- method$fun(x, control = control)
-    
+
     ser_permutation(ser_permutation_vector(order, method = method$name))
   }
 
@@ -48,11 +48,11 @@ seriate.dist <- function(x, method = "ARSA", control = NULL, ...) {
 ## of the elements on the ellipse is returned (see Chen 2002).
 seriate_dist_chen <- function(x, control = NULL){
   .get_parameters(control, NULL)
-  
+
   x <- as.matrix(x)
-  
+
   rank <- qr(x)$rank
-  
+
   ## find the first correlation matrix of rank 2
   n <- 0
   while(rank > 2){
@@ -60,21 +60,21 @@ seriate_dist_chen <- function(x, control = NULL){
     n <- n + 1
     rank <- qr(x)$rank
   }
-  
+
   ## project the matrix on the first 2 eigenvectors
   e <- eigen(x)$vectors[,1:2]
-  
+
   ## extract the order
   ## chen says that he uses the one of the two possible cuts
   ## that separate the points at rank 1. Since the points just
   ## separate further towards right and left, cutting on the vertical
   ## axis of the ellipse yields the same result.
-  
+
   right <- which(e[,1] >= 0)
   right <- right[order(e[right,2], decreasing = TRUE)]
   left <- which(e[,1] < 0)
   left <- left[order(e[left,2])]
-  
+
   o <- c(right,left)
   names(o) <- labels(x)[o]
   o
@@ -85,13 +85,17 @@ seriate_dist_chen <- function(x, control = NULL){
 seriate_dist_tsp <- function(x, control = NULL){
   ## add a dummy city for cutting
   tsp <- insert_dummy(TSP(x), n = 1, label = "cut_here")
-  
-  #    if(is.null(control)) control <- list(method="nearest_insertion")
-  if(is.null(control)) control <- list(method="farthest_insertion")
-  
+
+  if(is.null(control))
+    control <- list(
+      method="arbitrary insertion",
+      rep = 10,
+      two_opt = TRUE
+    )
+
   tour <- solve_TSP(tsp, method = control$method,
     control = control$control)
-  
+
   o <- cut_tour(tour, cut = "cut_here", exclude_cut = TRUE)
   names(o) <- labels(x)[o]
   o
@@ -103,21 +107,21 @@ seriate_dist_mds <- function(x, control = NULL){
   control <- .get_parameters(control, list(
     method = "cmdscale"
   ))
-  
+
   if(control$method == "cmdscale" ) {
     sc <- cmdscale(x, k=1)
     return(order(sc[,1]))
-    
+
   }else if(control$method == "isoMDS"){
     sc <- MASS::isoMDS(x+1e-6, trace = FALSE, k=1)
     return(order(sc$points[,1]))
-    
+
   }else if(control$method == "sammon") {
     sc <- MASS::sammon(x+1e-6, trace = FALSE, k=1)
     return(order(sc$points[,1]))
-    
+
   }else stop("unknown method")
-  
+
 }
 
 seriate_dist_mds_metric <- function(x, control = NULL)
@@ -128,9 +132,9 @@ seriate_dist_mds_nonmetric <- function(x, control = NULL)
 ## Angle between the first 2 PCS. Fiendly (2002)
 seriate_dist_angle <- function(x, control = NULL) {
   .get_parameters(control, NULL)
-  
+
   sc <- cmdscale(x, k=2)
-  o <- .order_angle(sc)
+  .order_angle(sc)
 }
 
 
@@ -140,7 +144,7 @@ seriate_dist_angle <- function(x, control = NULL) {
     hclust = NULL,
     method = "average"
     ))
-  
+
   if(!is.null(control$hclust)) return(control$hclust)
   return(hclust(d, method = control$method))
 }
@@ -152,6 +156,8 @@ seriate_dist_hc_average <- function(x, control = NULL)
   .hclust_helper(x, control=list(method="average"))
 seriate_dist_hc_complete <- function(x, control = NULL)
   .hclust_helper(x, control=list(method="complete"))
+seriate_dist_hc_ward <- function(x, control = NULL)
+  .hclust_helper(x, control=list(method="ward.D2"))
 
 ## workhorses are in seriation.hclust
 seriate_dist_gw <- function(x, control = NULL)
@@ -162,6 +168,8 @@ seriate_dist_gw_average <- function(x, control = NULL)
   reorder(seriate_dist_hc_average(x, control), x, method="GW")
 seriate_dist_gw_complete <- function(x, control = NULL)
   reorder(seriate_dist_hc_complete(x, control), x, method="GW")
+seriate_dist_gw_ward <- function(x, control = NULL)
+  reorder(seriate_dist_hc_ward(x, control), x, method="GW")
 
 
 seriate_dist_olo <- function(x, control = NULL)
@@ -172,6 +180,8 @@ seriate_dist_olo_average <- function(x, control = NULL)
   reorder(seriate_dist_hc_average(x, control), x, method="OLO")
 seriate_dist_olo_complete <- function(x, control = NULL)
   reorder(seriate_dist_hc_complete(x, control), x, method="OLO")
+seriate_dist_olo_ward <- function(x, control = NULL)
+  reorder(seriate_dist_hc_ward(x, control), x, method="OLO")
 
 ## brusco: simulated annealing for anti-robinson
 seriate_dist_arsa <- function(x, control = NULL) {
@@ -181,7 +191,7 @@ seriate_dist_arsa <- function(x, control = NULL) {
     nreps = 1L,
     verbose = FALSE
   ))
-  
+
   A <- as.matrix(x)
   # SUBROUTINE arsa(N, A, COOL, TMIN, NREPS, IPERM, R1, R2, D, U,
   #      S, T, SB, verbose)
@@ -194,19 +204,19 @@ seriate_dist_arsa <- function(x, control = NULL) {
   S <- integer(N)
   T <- integer(100*N)
   SB <- integer(N)
-  
+
   ret <- .Fortran("arsa", N, A, param$cool, param$tmin, param$nreps, IPERM,
     R1, R2, D, U, S, T, SB, param$verbose, PACKAGE="seriation")
-  
+
   o <- ret[[6]]
   names(o) <- labels(x)[o]
-  
+
   ### ARSA returns all 0's in some cases
   if(all(o == 0)) {
     o <- 1:N
     warning("ARSA has returned an invalid permutation vector! Check the supplied dissimilarity matrix.")
   }
-  
+
   o
 }
 
@@ -217,10 +227,10 @@ seriate_dist_bburcg <- function(x, control = NULL) {
     eps = 1e-7,
     verbose = FALSE
   ))
-  
+
   A <- as.matrix(x)
   N <- ncol(A)
-  
+
   # SUBROUTINE bburcg(N, A, EPS, X, Q, D, DD, S, UNSEL, IVERB)
   X <- integer(N)
   Q <- integer(N)
@@ -228,10 +238,10 @@ seriate_dist_bburcg <- function(x, control = NULL) {
   DD <- integer(N*N*N)
   S <- integer(N)
   UNSEL <- integer(N)
-  
+
   ret <- .Fortran("bburcg", N, A, param$eps, X, Q, D, DD, S, UNSEL,
     param$verbose)
-  
+
   o <- ret[[4]]
   names(o) <- labels(x)[o]
   o
@@ -244,10 +254,10 @@ seriate_dist_bbwrcg <- function(x, control = NULL) {
     eps = 1e-7,
     verbose = FALSE
   ))
-  
+
   A <- as.matrix(x)
   N <- ncol(A)
-  
+
   # SUBROUTINE bbwrcg(N, A, EPS, X, Q, D, DD, S, UNSEL, IVERB)
   X <- integer(N)
   Q <- integer(N)
@@ -255,24 +265,26 @@ seriate_dist_bbwrcg <- function(x, control = NULL) {
   DD <- double(N*N*N)
   S <- integer(N)
   UNSEL <- integer(N)
-  
+
   ret <- .Fortran("bbwrcg", N, A, param$eps, X, Q, D, DD, S, UNSEL,
     param$verbose)
-  
+
   o <- ret[[4]]
   names(o) <- labels(x)[o]
   o
 }
 
 seriate_dist_identity <- function(x, control = NULL) {
-  param <- .get_parameters(control, NULL)
+  #param <- .get_parameters(control, NULL)
+  .get_parameters(control, NULL)
+
   o <- 1:attr(x, "Size")
   names(o) <- labels(x)
   o
 }
 
 seriate_dist_random <- function(x, control = NULL) {
-  param <- .get_parameters(control, NULL)
+  #param <- .get_parameters(control, NULL)
   o <- 1:attr(x, "Size")
   names(o) <- labels(x)
   sample(o)
@@ -283,18 +295,19 @@ seriate_dist_random <- function(x, control = NULL) {
 ## Proceedings of the 2002 International Joint Conference on
 ## Neural Networks, 2002. IJCNN '02. (Volume:3)
 seriate_dist_VAT <- function(x, control = NULL) {
-  param <- .get_parameters(control, NULL)
-  
+  #param <- .get_parameters(control, NULL)
+  .get_parameters(control, NULL)
+
   D <- as.matrix(x)
   N <- nrow(D)
   P <- rep(NA_integer_, N)
   I <- rep(FALSE, N)
   ### J is !I
-  
+
   i <- which(D == max(D, na.rm = TRUE), arr.ind = TRUE)[1,1]
   P[1] <- i
   I[i] <- TRUE
-  
+
   for(r in 2:N) {
     D2 <- D[I,!I, drop=FALSE]
     j <- which(D2 == min(D2, na.rm = TRUE), arr.ind = TRUE)[1,2]
@@ -302,48 +315,52 @@ seriate_dist_VAT <- function(x, control = NULL) {
     P[r] <- j
     I[j] <- TRUE
   }
-  
+
   names(P) <- labels(x)[P]
   P
 }
 
 
-## spectral seriation
+## Spectral Seriation
 ## Ding, C. and Xiaofeng He (2004): Linearized cluster assignment via
 ## spectral orderingProceedings of the Twenty-first.
 ## International Conference on Machine learning (ICML '04)
 
+## Minimizes: sum_{i,j} (i-j)^2 * d_{pi_i,pi_j}
+
 seriate_dist_spectral <- function(x, control = NULL) {
-  param <- .get_parameters(control, NULL)
-  
+  #param <- .get_parameters(control, NULL)
+  .get_parameters(control, NULL)
+
   ### calculate Laplacian
-  A <- 1/(1+as.matrix(x))
-  D <- diag(rowSums(A))
-  L <- D - A
-  
-  e <- eigen(L)
-  fielder <- e$vectors[,ncol(A)-1L]
+  W <- 1/(1+as.matrix(x))
+  D <- diag(rowSums(W))
+  L <- D - W
+
+  ## Fielder vector q1 is eigenvector with the smallest eigenvalue
+  ## eigen reports eigenvectors/values in decreasing order
+  q <- eigen(L)
+  fielder <- q$vectors[,ncol(W)-1L]
   o <- order(fielder)
   names(o) <- names(x)[o]
   o
 }
 
-### FIXME?
 seriate_dist_spectral_norm <- function(x, control = NULL) {
-  param <- .get_parameters(control, NULL)
-  
+  #param <- .get_parameters(control, NULL)
+  .get_parameters(control, NULL)
+
   ### calculate normalized Laplacian
-  A <- 1/(1+as.matrix(x))
-  D_sqrt<- diag(rowSums(A))^.5
-  #I <- diag(nrow(A))
-  #L <- I - D_sqrt %*% A %*% D_sqrt
-  L <- D_sqrt %*% A %*% D_sqrt
-  
+  W <- 1/(1+as.matrix(x))
+  D_sqrt<- diag(rowSums(1/W^.5))
+  L <- D_sqrt %*% W %*% D_sqrt
+
   z <- eigen(L)$vectors
   q <- D_sqrt %*% z
-  #o <- order(q[,ncol(q)-1L])
-  o <- order(q[,2L])
-  #pimage(x, o)
+
+  ## look for the vector with the largest eigenvalue
+  largest_ev <- q[,2L]
+  o <- order(largest_ev)
   names(o) <- names(x)[o]
   o
 }
@@ -353,9 +370,9 @@ seriate_dist_spectral_norm <- function(x, control = NULL) {
 ## Weight matrix
 ## pimage(create_x(n=150, sigma=20, verbose=TRUE))
 create_W <- function(n, sigma, verbose=FALSE) {
-  w <- function(i, j, n, sigma) exp(-1*(i-j)^2/n/sigma) 
+  w <- function(i, j, n, sigma) exp(-1*(i-j)^2/n/sigma)
   W <- outer(1:n, 1:n, FUN = w, n=n, sigma=sigma)
-  
+
   ## make doubly stochastic
   for(i in 1:1000) {
     #cat(i, ".")
@@ -363,10 +380,10 @@ create_W <- function(n, sigma, verbose=FALSE) {
     W <- sweep(W, MARGIN = 2, STATS = colSums(W), "/")
     if(round(rowSums(W), 5) == 1 && round(colSums(W), 5) == 1) break
   }
-  
+
   if(verbose) cat("It took", i, "iterations to make W doubly stochastic!\n")
   if(i >999) warning("Weight matrix did not converge to doubly stochastic in 1000 itermation!")
-  W 
+  W
 }
 
 ## SPIN: Neighborhood algorithms
@@ -377,59 +394,59 @@ seriate_dist_SPIN <- function(x, control = NULL) {
     W_function = NULL,
     verbose = FALSE
   ))
-  
-  W_function <- if(is.null(param$W_function)) create_W else param$W_function 
+
+  W_function <- if(is.null(param$W_function)) create_W else param$W_function
   sigma <- param$sigma
   step <- param$step
   verbose <- param$verbose
-  
+
   D <- as.matrix(x)
   n <- nrow(D)
-  
-  ## weight matrix  
+
+  ## weight matrix
   W <- W_orig <- W_function(n, sigma[1], verbose)
-  
+
   energy_best <- Inf
-  
+
   for(i in 1:(length(sigma)*step)) {
     if(verbose) cat("Iteration", i, "... ")
-    
+
     M <- D %*% W
-    
+
     ## heuristic for the linear assignment problem
     ## (second argument to order breakes ties randomly)
     P <- permutation_vector2matrix(
       order(apply(M, MARGIN = 1, which.min), sample(1:n)))
     #if(verbose) print(table(apply(M, MARGIN = 1, which.min)))
-    
+
     energy_new <- sum(diag(P %*% M))
-    if(verbose) cat("best energy:", energy_best, 
+    if(verbose) cat("best energy:", energy_best,
       "new energy: ", energy_new, "\n")
-    
+
     ## was energy improved?
-    if(energy_new < energy_best) { 
+    if(energy_new < energy_best) {
       energy_best <- energy_new
       P_best <- P
     }
-    
+
     ## adapt sigma
     if(!(i %% step) && i != length(sigma)*step) {
       s <- sigma[i/step+1]
       if(verbose) cat("\nReducing sigma to:", s, "\n")
-      
+
       W_orig <- W_function(n, s, verbose)
-      
-      
+
+
       ## recalculate best energy
       W <- crossprod(P, W_orig) ### t(P) %*% W
       M <- D %*% W
       energy_best <- sum(diag(P %*% M))
       if(verbose) cat("best energy is now:", energy_best, "\n\n")
     }else {
-      W_m <- crossprod(P, W_orig) ### t(P) %*% W
+      W <- crossprod(P, W_orig) ### t(P) %*% W
     }
   }
-  
+
   if(verbose) cat("Best Energy:", energy_best, "\n")
   o <- permutation_matrix2vector(P_best)
   names(o) <- names(x)[o]
@@ -446,76 +463,93 @@ seriate_dist_SPIN_STS <- function(x, control = NULL) {
     X = function(n) 1:n - (n+1)/2,
     verbose = FALSE
   ))
-  
+
   step <- param$step
   verbose <- param$verbose
   nstart <- param$nstart
   X <- param$X
-  
+
   D <- as.matrix(x)
   n <- nrow(D)
-  
+
   ## X for weights W = X %*% t(X) (colunm vector)
   if(is.function(X)) X <- X(n)
   if(!is.numeric(X) || length(X) != n) stop("Invalid weight vector X.")
   W <- tcrossprod(X) ## X %*% t(X)
-  
+
   .STS_run <- function() {
     if(verbose) cat("\nStarting new run\n")
-    
+
     ## start with random permutation
     o_best <- o <- sample(1:n)
     #P_best <- P <- permutation_vector2matrix(o)
     #X_current <- crossprod(P, X)
     X_current <- X[o]
-    #energy_best <- sum(diag(P %*% D %*% t(P) %*% W)) 
-    energy_best <- sum(diag(D[o,o] %*% W)) 
-    
+    #energy_best <- sum(diag(P %*% D %*% t(P) %*% W))
+    energy_best <- sum(diag(D[o,o] %*% W))
+
     for(i in 1:step) {
       if(verbose) cat("Iteration", i, "... ")
-      
+
       ## permutation matrix that orders S in descending order (break ties)
       S <- D %*% X_current
       o <- order(S, sample(1:n), decreasing = TRUE)
       #P <- permutation_vector2matrix(o)
       #X_current <- crossprod(P, X) ## t(P) %*% X
       X_current <- X[o] ## t(P) %*% X
-      
+
       ## calculate energy F(P)
-      #energy_new <- sum(diag(P %*% D %*% t(P) %*% W)) 
-      energy_new <- sum(diag(D[o,o] %*% W)) 
-      if(verbose) cat("best energy:", energy_best, 
+      #energy_new <- sum(diag(P %*% D %*% t(P) %*% W))
+      energy_new <- sum(diag(D[o,o] %*% W))
+      if(verbose) cat("best energy:", energy_best,
         "new energy: ", energy_new)
-      
+
       ## was energy improved?
-      if(energy_new < energy_best) { 
+      if(energy_new < energy_best) {
         energy_best <- energy_new
         #P_best <- P
         o_best <- o
-        if(verbose) cat(" - update") 
+        if(verbose) cat(" - update")
       }
-      
-      if(verbose) cat("\n") 
-      
+
+      if(verbose) cat("\n")
+
     }
-    
+
     if(verbose) cat("Best Energy:", energy_best, "\n")
-    
+
     #o <- permutation_matrix2vector(P_best)
     o <- o_best
     attr(o, "energy") <- energy_best
     o
   }
-  
+
   res <- replicate(nstart, .STS_run(), simplify = FALSE)
   energy <- sapply(res, attr, "energy")
-  
+
   if(verbose) cat("Overall best Energy:", min(energy), "\n")
   o <- res[[which.min(energy)]]
   names(o) <- names(x)[o]
   o
 }
 
+## QAP 2SUM seriation
+seriate_dist_2SUM <- function(x, control = NULL) {
+  ## param are passed on to QAP
+
+  do.call(qap::qap, c(list(A = .A_2SUM(attr(x, "Size")),
+    B = 1/(1+as.matrix(x))), control))
+}
+
+## QAP Linear seriation
+seriate_dist_LS <- function(x, control = NULL) {
+  ## param are passed on to QAP
+
+  do.call(qap::qap, c(list(A = .A_LS(attr(x, "Size")),
+    B = as.matrix(x)), control))
+}
+
+
 
 set_seriation_method("dist", "Identity", seriate_dist_identity,
   "Identity permutation")
@@ -555,6 +589,8 @@ set_seriation_method("dist", "HC_complete", seriate_dist_hc_complete,
   "Hierarchical clustering (complete link)")
 set_seriation_method("dist", "HC_average", seriate_dist_hc_average,
   "Hierarchical clustering (avg. link)")
+set_seriation_method("dist", "HC_ward", seriate_dist_hc_ward,
+  "Hierarchical clustering (Ward's method)")
 
 set_seriation_method("dist", "GW", seriate_dist_gw,
   "Hierarchical clustering reordered by Gruvaeus and Wainer heuristic")
@@ -564,6 +600,9 @@ set_seriation_method("dist", "GW_average", seriate_dist_gw_average,
   "Hierarchical clustering (avg. link) reordered by Gruvaeus and Wainer heuristic")
 set_seriation_method("dist", "GW_complete", seriate_dist_gw_complete,
   "Hierarchical clustering (complete link) reordered by Gruvaeus and Wainer heuristic")
+set_seriation_method("dist", "GW_ward", seriate_dist_gw_ward,
+  "Hierarchical clustering (Ward's method) reordered by Gruvaeus and Wainer heuristic")
+
 
 set_seriation_method("dist", "OLO", seriate_dist_olo,
   "Hierarchical clustering (single link) with optimal leaf ordering")
@@ -573,6 +612,8 @@ set_seriation_method("dist", "OLO_average", seriate_dist_olo_average,
   "Hierarchical clustering (avg. link) with optimal leaf ordering")
 set_seriation_method("dist", "OLO_complete", seriate_dist_olo_complete,
   "Hierarchical clustering (complete link) with optimal leaf ordering")
+set_seriation_method("dist", "OLO_ward", seriate_dist_olo_ward,
+  "Hierarchical clustering (Ward's method) with optimal leaf ordering")
 
 set_seriation_method("dist", "VAT", seriate_dist_VAT,
   "Visual assesment of clustering tendency (VAT)")
@@ -585,4 +626,9 @@ set_seriation_method("dist", "Spectral_norm", seriate_dist_spectral_norm,
 set_seriation_method("dist", "SPIN_NH", seriate_dist_SPIN,
   "SPIN (Neighborhood algorithm)")
 set_seriation_method("dist", "SPIN_STS", seriate_dist_SPIN_STS,
-  "SPIN (Side-to-Side algorithm)")
\ No newline at end of file
+  "SPIN (Side-to-Side algorithm)")
+
+set_seriation_method("dist", "QAP_2SUM", seriate_dist_2SUM,
+  "2-SUM (QAP)")
+set_seriation_method("dist", "QAP_LS", seriate_dist_LS,
+  "Linear Seriation (QAP)")
